@@ -95,6 +95,9 @@ class _config(_base_config):
     loading_color = param.Color(default='#c3c3c3', doc="""
         Color of the loading indicator.""")
 
+    loading_max_height = param.Integer(default=400, doc="""
+        Maximum height of the loading indicator.""")
+
     safe_embed = param.Boolean(default=False, doc="""
         Ensure all bokeh property changes trigger events which are
         embedded. Useful when only partial updates are made in an
@@ -157,6 +160,9 @@ class _config(_base_config):
         default=None, allow_None=True, objects=[], doc="""
         Select between a list of authentification providers.""")
 
+    _oauth_expiry = param.Number(default=1, bounds=(0, None), doc="""
+        Expiry of the OAuth cookie in number of days.""")
+
     _oauth_key = param.String(default=None, doc="""
         A client key to provide to the OAuth provider.""")
 
@@ -301,6 +307,11 @@ class _config(_base_config):
         provider = os.environ.get('PANEL_OAUTH_PROVIDER', _config._oauth_provider)
         return provider.lower() if provider else None
 
+    @property
+    def oauth_expiry(self):
+        provider = os.environ.get('PANEL_OAUTH_EXPIRY', _config._oauth_expiry)
+        return float(provider)
+
     @property
     def oauth_key(self):
         return os.environ.get('PANEL_OAUTH_KEY', _config._oauth_key)
@@ -388,7 +399,10 @@ class panel_extension(_pyviz_extension):
     _loaded_extensions = []
 
     def __call__(self, *args, **params):
-        # Abort if IPython not found
+        # Load ipywidget support if requested in environment variable
+        if 'ipywidgets' in sys.modules and os.environ.get("PANEL_IPYWIDGET"):
+            args += ('ipywidgets',)
+    
         for arg in args:
             if arg not in self._imports:
                 self.param.warning('%s extension not recognized and '
@@ -434,6 +448,7 @@ class panel_extension(_pyviz_extension):
             else:
                 hv.Store.current_backend = backend
 
+        # Abort if IPython not found
         try:
             ip = params.pop('ip', None) or get_ipython() # noqa (get_ipython)
         except Exception:
