@@ -24,7 +24,7 @@ import bokeh.command.util
 
 # Bokeh imports
 from bokeh.application import Application as BkApplication
-from bokeh.application.handlers.code import CodeHandler
+from bokeh.application.handlers.code import CodeHandler, _monkeypatch_io, patch_curdoc
 from bokeh.application.handlers.function import FunctionHandler
 from bokeh.command.util import build_single_handler_application
 from bokeh.core.templates import AUTOLOAD_JS
@@ -34,9 +34,11 @@ from bokeh.embed.elements import html_page_for_render_items, script_for_render_i
 from bokeh.embed.util import RenderItem
 from bokeh.io import curdoc
 from bokeh.server.server import Server
-from bokeh.server.urls import per_app_patterns
+from bokeh.server.urls import per_app_patterns, toplevel_patterns
 from bokeh.server.views.autoload_js_handler import AutoloadJsHandler as BkAutoloadJsHandler
 from bokeh.server.views.doc_handler import DocHandler as BkDocHandler
+from bokeh.server.views.root_handler import RootHandler as BkRootHandler
+from bokeh.server.views.static_handler import StaticHandler
 
 # Tornado imports
 from tornado.ioloop import IOLoop
@@ -98,13 +100,13 @@ def async_execute(func):
         unlock = not func.func.lock
     else:
         unlock = not getattr(func, 'lock', False)
-    if unlock:
-        @wraps(func)
-        async def wrapper(*args, **kw):
+    curdoc = state.curdoc
+    @wraps(func)
+    async def wrapper(*args, **kw):
+        with set_curdoc(curdoc):
             return await func(*args, **kw)
+    if unlock:
         wrapper.nolock = True
-    else:
-        wrapper = func
     state.curdoc.add_next_tick_callback(wrapper)
 
 param.parameterized.async_executor = async_execute
@@ -146,13 +148,13 @@ def server_html_page_for_session(session, resources, title, template=BASE_TEMPLA
     if template_variables is None:
         template_variables = {}
 
-    bundle = bundle_resources(resources)
+    bundle = bundle_resources(session.document.roots, resources)
     return html_page_for_render_items(bundle, {}, [render_item], title,
         template=template, template_variables=template_variables)
 
-def autoload_js_script(resources, token, element_id, app_path, absolute_url):
+def autoload_js_script(doc, resources, token, element_id, app_path, absolute_url):
     resources = Resources.from_bokeh(resources)
-    bundle = bundle_resources(resources)
+    bundle = bundle_resources(doc.roots, resources)
 
     render_items = [RenderItem(token=token, elementid=element_id, use_for_title=False)]
     bundle.add(Script(script_for_render_items({}, render_items, app_path=app_path, absolute_url=absolute_url)))
@@ -167,6 +169,12 @@ class Application(BkApplication):
             cb(session_context)
         await super().on_session_created(session_context)
 
+    def initialize_document(self, doc):
+        super().initialize_document(doc)
+        if doc in state._templates:
+            template = state._templates[doc]
+            template.server_doc(title=template.title, location=True, doc=doc)
+
 bokeh.command.util.Application = Application
 
 
@@ -180,6 +188,13 @@ class SessionPrefixHandler:
         base_url = urljoin('/', prefix)
         rel_path = '/'.join(['..'] * self.application_context._url.strip('/').count('/'))
         old_url, old_rel = state.base_url, state.rel_path
+
+        # Handle autoload.js absolute paths
+        abs_url = self.get_argument('bokeh-absolute-url', default=None)
+        if abs_url is not None:
+            app_path = self.get_argument('bokeh-app-path', default=None)
+            rel_path = abs_url.replace(app_path, '')
+
         with edit_readonly(state):
             state.base_url = base_url
             state.rel_path = rel_path
@@ -197,12 +212,16 @@ class DocHandler(BkDocHandler, SessionPrefixHandler):
     async def get(self, *args, **kwargs):
         with self._session_prefix():
             session = await self.get_session()
-            resources = Resources.from_bokeh(self.application.resources())
-            page = server_html_page_for_session(
-                session, resources=resources, title=session.document.title,
-                template=session.document.template,
-                template_variables=session.document.template_variables
-            )
+            state.curdoc = session.document
+            try:
+                resources = Resources.from_bokeh(self.application.resources())
+                page = server_html_page_for_session(
+                    session, resources=resources, title=session.document.title,
+                    template=session.document.template,
+                    template_variables=session.document.template_variables
+                )
+            finally:
+                state.curdoc = None
         self.set_header("Content-Type", 'text/html')
         self.write(page)
 
@@ -230,14 +249,34 @@ class AutoloadJsHandler(BkAutoloadJsHandler, SessionPrefixHandler):
 
         with self._session_prefix():
             session = await self.get_session()
-            resources = Resources.from_bokeh(self.application.resources(server_url))
-            js = autoload_js_script(resources, session.token, element_id, app_path, absolute_url)
+            state.curdoc = session.document
+            try:
+                resources = Resources.from_bokeh(self.application.resources(server_url))
+                js = autoload_js_script(
+                    session.document, resources, session.token, element_id,
+                    app_path, absolute_url
+                )
+            finally:
+                state.curdoc = None
 
         self.set_header("Content-Type", 'application/javascript')
         self.write(js)
 
 per_app_patterns[3] = (r'/autoload.js', AutoloadJsHandler)
 
+class RootHandler(BkRootHandler):
+
+    @authenticated
+    async def get(self, *args, **kwargs):
+        if self.index and not self.index.endswith('.html'):
+            prefix = "" if self.prefix is None else self.prefix
+            redirect_to = prefix + '.'.join(self.index.split('.')[:-1])
+            self.redirect(redirect_to)
+        await super().get(*args, **kwargs)
+
+toplevel_patterns[0] = (r'/?', RootHandler)
+bokeh.server.tornado.RootHandler = RootHandler
+
 def modify_document(self, doc):
     from bokeh.io.doc import set_curdoc as bk_set_curdoc
     from ..config import config
@@ -261,11 +300,10 @@ def modify_document(self, doc):
     # stored modules are used to provide correct paths to
     # custom models resolver.
     sys.modules[module.__name__] = module
-    doc._modules.append(module)
+    doc.modules._modules.append(module)
 
     old_doc = curdoc()
     bk_set_curdoc(doc)
-    old_io = self._monkeypatch_io()
 
     if config.autoreload:
         set_curdoc(doc)
@@ -274,6 +312,12 @@ def modify_document(self, doc):
     try:
         def post_check():
             newdoc = curdoc()
+            # Do not let curdoc track modules when autoreload is enabled
+            # otherwise it will erroneously complain that there is
+            # a memory leak
+            if config.autoreload:
+                newdoc.modules._modules = []
+
             # script is supposed to edit the doc not replace it
             if newdoc is not doc:
                 raise RuntimeError("%s at '%s' replaced the output document" % (self._origin, self._runner.path))
@@ -284,7 +328,11 @@ def modify_document(self, doc):
 
             # Clean up
             del sys.modules[module.__name__]
-            doc._modules.remove(module)
+
+            if hasattr(doc, 'modules'):
+                doc.modules._modules.remove(module)
+            else:
+                doc._modules.remove(module)
             bokeh.application.handlers.code_runner.handle_exception = handle_exception
             tb = html.escape(traceback.format_exc())
 
@@ -296,13 +344,27 @@ def modify_document(self, doc):
 
         if config.autoreload:
             bokeh.application.handlers.code_runner.handle_exception = handle_exception
-        self._runner.run(module, post_check)
+        with _monkeypatch_io(self._loggers):
+            with patch_curdoc(doc):
+                self._runner.run(module, post_check)
     finally:
-        self._unmonkeypatch_io(old_io)
         bk_set_curdoc(old_doc)
 
 CodeHandler.modify_document = modify_document
 
+# Copied from bokeh 2.4.0, to fix directly in bokeh at some point.
+def create_static_handler(prefix, key, app):
+    # patch
+    key = '/__patchedroot' if key == '/' else key
+
+    route = prefix
+    route += "/static/(.*)" if key == "/" else key + "/static/(.*)"
+    if app.static_path is not None:
+        return (route, StaticFileHandler, {"path" : app.static_path})
+    return (route, StaticHandler, {})
+
+bokeh.server.tornado.create_static_handler = create_static_handler
+
 #---------------------------------------------------------------------
 # Public API
 #---------------------------------------------------------------------
@@ -369,9 +431,9 @@ def unlocked():
         return
     connections = curdoc.session_context.session._subscribed_connections
 
-    hold = curdoc._hold
+    hold = curdoc.callbacks.hold_value
     if hold:
-        old_events = list(curdoc._held_events)
+        old_events = list(curdoc.callbacks._held_events)
     else:
         old_events = []
         curdoc.hold()
@@ -383,7 +445,7 @@ def unlocked():
             if hasattr(socket, 'write_lock') and socket.write_lock._block._value == 0:
                 state._locks.add(socket)
             locked = socket in state._locks
-            for event in curdoc._held_events:
+            for event in curdoc.callbacks._held_events:
                 if (isinstance(event, ModelChangedEvent) and event not in old_events
                     and hasattr(socket, 'write_message') and not locked):
                     msg = conn.protocol.create('PATCH-DOC', [event])
@@ -395,7 +457,7 @@ def unlocked():
                         WebSocketHandler.write_message(socket, payload, binary=True)
                 elif event not in events:
                     events.append(event)
-        curdoc._held_events = events
+        curdoc.callbacks._held_events = events
     finally:
         if not hold:
             curdoc.unhold()
@@ -430,9 +492,9 @@ def serve(panels, port=0, address=None, websocket_origin=None, loop=None,
       If None, "localhost" is used.
     loop : tornado.ioloop.IOLoop (optional, default=IOLoop.current())
       The tornado IOLoop to run the Server on
-    show : boolean (optional, default=False)
+    show : boolean (optional, default=True)
       Whether to open the server in a new browser tab on start
-    start : boolean(optional, default=False)
+    start : boolean(optional, default=True)
       Whether to start the Server
     title: str or {str: str} (optional, default=None)
       An HTML title for the application or a dictionary mapping
@@ -458,6 +520,8 @@ def serve(panels, port=0, address=None, websocket_origin=None, loop=None,
         server = StoppableThread(
             target=get_server, io_loop=loop, args=(panels,), kwargs=kwargs
         )
+        server_id = kwargs.get('server_id', uuid.uuid4().hex)
+        state._threads[server_id] = server
         server.start()
     else:
         server = get_server(panels, **kwargs)
