@@ -1,10 +1,11 @@
 """
 Defines custom VegaPlot bokeh model to render Vega json plots.
 """
-from bokeh.core.properties import Dict, String, Any, Instance
+from bokeh.core.properties import Any, Dict, Instance, Nullable, String
 from bokeh.models import LayoutDOM, ColumnDataSource
 
-from ..util import classproperty, bundled_files
+from ..io.resources import bundled_files
+from ..util import classproperty
 
 
 class VegaPlot(LayoutDOM):
@@ -32,15 +33,14 @@ class VegaPlot(LayoutDOM):
         }
 
     __js_require__ = {
-        'baseUrl': 'https://cdn.jsdelivr.net/npm/',
         'paths': {
-            "vega-embed":  "vega-embed@6/build/vega-embed.min",
-            "vega-lite": "vega-lite@4/build/vega-lite.min",
-            "vega": "vega@5/build/vega.min"
+            "vega-embed":  "https://cdn.jsdelivr.net/npm/vega-embed@6/build/vega-embed.min",
+            "vega-lite": "https://cdn.jsdelivr.net/npm/vega-lite@4/build/vega-lite.min",
+            "vega": "https://cdn.jsdelivr.net/npm/vega@5/build/vega.min"
         },
         'exports': {'vega-embed': 'vegaEmbed', 'vega': 'vega', 'vega-lite': 'vl'}
     }
 
-    data = Dict(String, Any)
+    data = Nullable(Dict(String, Any))
 
     data_sources = Dict(String, Instance(ColumnDataSource))
