@@ -1,8 +1,10 @@
 import * as p from "@bokehjs/core/properties"
+import {isArray} from "@bokehjs/core/util/types"
 import {HTMLBox, HTMLBoxView} from "@bokehjs/models/layouts/html_box"
 
 export class VegaPlotView extends HTMLBoxView {
   model: VegaPlot
+  vega_view: any
   _connected: string[]
 
   connect_signals(): void {
@@ -29,7 +31,7 @@ export class VegaPlotView extends HTMLBoxView {
       const cds = this.model.data_sources[ds];
       const data: any = []
       const columns = cds.columns()
-      for (let i = 0; i < cds.data[columns[0]].length; i++) {
+      for (let i = 0; i < cds.get_length(); i++) {
         const item: any = {}
         for (const column of columns) {
           item[column] = cds.data[column][i]
@@ -47,16 +49,18 @@ export class VegaPlotView extends HTMLBoxView {
   }
 
   _plot(): void {
-    if (!this.model.data || !(window as any).vegaEmbed)
+    const data = this.model.data
+    if ((data == null) || !(window as any).vegaEmbed)
       return
     if (this.model.data_sources && (Object.keys(this.model.data_sources).length > 0)) {
       const datasets = this._fetch_datasets()
       if ('data' in datasets) {
-        this.model.data.data['values'] = datasets['data']
+        data.data['values'] = datasets['data']
         delete datasets['data']
       }
-      if (this.model.data.data !== undefined) {
-        for (const d of this.model.data.data) {
+      if (data.data != null) {
+        const data_objs = isArray(data.data) ? data.data : [data.data]
+        for (const d of data_objs) {
           if (d.name in datasets) {
             d['values'] = datasets[d.name]
             delete datasets[d.name]
@@ -65,7 +69,33 @@ export class VegaPlotView extends HTMLBoxView {
       }
       this.model.data['datasets'] = datasets
     }
-    (window as any).vegaEmbed(this.el, this.model.data, {actions: false})
+    (window as any).vegaEmbed(this.el, this.model.data, {actions: false}).then((result: any) => {
+      this.vega_view = result.view
+      this.relayout()
+      if (this.vega_view._viewHeight <= 0 || this.vega_view._viewWidth <= 0) {
+	(window as any).dispatchEvent(new Event('resize'));
+      }
+    })
+  }
+
+  relayout(): void {
+    this.update_layout()
+    this.compute_layout()
+    if (this.root !== this)
+      this.invalidate_layout()
+    else if ((this as any)._parent != undefined) // HACK: Support ReactiveHTML
+      (this as any)._parent.invalidate_layout()
+  }
+
+  box_sizing(): any {
+    const sizing = super.box_sizing()
+    if (this.vega_view != null) {
+      if (sizing.height_policy === "fixed")
+	sizing.height = this.vega_view._viewHeight
+      if (sizing.width_policy === "fixed")
+	sizing.width = this.vega_view._viewWidth
+    }
+    return sizing
   }
 }
 
@@ -91,9 +121,9 @@ export class VegaPlot extends HTMLBox {
   static init_VegaPlot(): void {
     this.prototype.default_view = VegaPlotView
 
-    this.define<VegaPlot.Props>({
-      data: [ p.Any         ],
-      data_sources: [ p.Any  ],
-    })
+    this.define<VegaPlot.Props>(({Any}) => ({
+      data:         [ Any, {} ],
+      data_sources: [ Any, {} ],
+    }))
   }
 }
