@@ -84,7 +84,6 @@ class DataFrame(HTML):
     DataFrame renders pandas, dask and streamz DataFrame types using
     their custom HTML repr. In the case of a streamz DataFrame the
     rendered data will update periodically.
-
     """
 
     bold_rows = param.Boolean(default=True, doc="""
@@ -104,6 +103,10 @@ class DataFrame(HTML):
     decimal = param.String(default='.', doc="""
         Character recognized as decimal separator, e.g. ',' in Europe.""")
 
+    escape = param.Boolean(default=True, doc="""
+        Whether or not to escape the dataframe HTML. For security reasons
+        the default value is True.""")
+
     float_format = param.Callable(default=None, doc="""
         Formatter function to apply to columns' elements if they are
         floats. The result of this function must be a unicode string.""")
@@ -152,7 +155,7 @@ class DataFrame(HTML):
 
     _rerender_params = [
         'object', '_object', 'bold_rows', 'border', 'classes',
-        'col_space', 'decimal', 'float_format', 'formatters',
+        'col_space', 'decimal', 'escape', 'float_format', 'formatters',
         'header', 'index', 'index_names', 'justify', 'max_rows',
         'max_cols', 'na_rep', 'render_links', 'show_dimensions',
         'sparsify', 'sizing_mode'
@@ -240,8 +243,8 @@ class Str(DivPaneBase):
 
     def _get_properties(self):
         properties = super()._get_properties()
-        if self.object is None:
-            text = ''
+        if self.object is None or (isinstance(self.object, str) and self.object == ''):
+            text = '<pre> </pre>'
         else:
             text = '<pre>'+str(self.object)+'</pre>'
         return dict(properties, text=escape(text))
