@@ -1,5 +1,16 @@
+from enum import Enum
 from collections import namedtuple
 
+class TextPosition(Enum):
+    LowerLeft = 0 
+    LowerRight = 1  
+    UpperLeft = 2 
+    UpperRight = 3  
+    LowerEdge = 4 
+    RightEdge = 5  
+    LeftEdge = 6  
+    UpperEdge = 7   
+
 SCALAR_MODE = namedtuple("SCALAR_MODE",
     "Default UsePointData UseCellData UsePointFieldData UseCellFieldData UseFieldData"
 )(0, 1, 2, 3, 4, 5)
