@@ -9,6 +9,9 @@ import zipfile
 from vtk.vtkCommonCore import vtkTypeUInt32Array, vtkTypeInt32Array
 from vtk.vtkFiltersGeometry import vtkCompositeDataGeometryFilter, vtkGeometryFilter
 from vtk.vtkRenderingCore import vtkColorTransferFunction
+from vtk.vtkCommonDataModel import vtkDataObject
+
+from .enums import TextPosition
 
 # -----------------------------------------------------------------------------
 # Python compatibility handling 2.6, 2.7, 3+
@@ -175,6 +178,7 @@ class SynchronizationContext():
         self.idRoot = id_root
         self.debugSerializers = debug
         self.debugAll = debug
+        self.annotations = {}
 
     def getReferenceId(self, instance):
         if not self.idRoot or (hasattr(instance, 'IsA') and instance.IsA('vtkCamera')):
@@ -182,6 +186,24 @@ class SynchronizationContext():
         else:
             return self.idRoot + getReferenceId(instance)
 
+    def addAnnotation(self, parent, prop, propId):
+        if prop.GetClassName() == "vtkCornerAnnotation":
+            annotation = {
+                "id": propId,
+                "viewport": parent.GetViewport(),
+                "fontSize": prop.GetLinearFontScaleFactor() * 2,
+                "fontFamily": prop.GetTextProperty().GetFontFamilyAsString(),
+                "color": prop.GetTextProperty().GetColor(),
+                **{pos.name: prop.GetText(pos.value) for pos in TextPosition}
+            }
+            if self.annotations is None:
+                self.annotations = {propId: annotation}
+            else:
+                self.annotations.update({propId: annotation})
+
+    def getAnnotations(self):
+        return list(self.annotations.values())
+
     def setIgnoreLastDependencies(self, force):
         self.ingoreLastDependencies = force
 
@@ -290,6 +312,9 @@ def serializeInstance(parent, instance, instanceId, context, depth):
 
 
 def initializeSerializers():
+    # Annotations
+    registerInstanceSerializer('vtkCornerAnnotation', annotationSerializer)
+
     # Actors/viewProps
     registerInstanceSerializer('vtkImageSlice', genericProp3DSerializer)
     registerInstanceSerializer('vtkVolume', genericProp3DSerializer)
@@ -310,6 +335,8 @@ def initializeSerializers():
         'vtkSmartVolumeMapper', genericVolumeMapperSerializer)
     registerInstanceSerializer(
         'vtkOpenGLImageSliceMapper', imageSliceMapperSerializer)
+    registerInstanceSerializer(
+        'vtkOpenGLGlyph3DMapper', glyph3DMapperSerializer)
 
     # LookupTables/TransferFunctions
     registerInstanceSerializer('vtkLookupTable', lookupTableSerializer)
@@ -512,6 +539,23 @@ def extractRequiredFields(extractedFields, parent, dataset, context, requestedFi
             arrayMeta['location'] = 'pointData'
             arrayMeta['registration'] = 'setScalars'
             extractedFields.append(arrayMeta)
+        
+    if parent.IsA("vtkGlyph3DMapper") and not context.serializeAllDataArrays:
+        scaleArrayName = parent.GetInputArrayInformation(parent.SCALE).Get(vtkDataObject.FIELD_NAME())
+        if scaleArrayName is not None and scaleArrayName not in [field['name'] for field in extractedFields]:
+            arrayMeta = getArrayDescription(dataset.GetPointData().GetAbstractArray(scaleArrayName), context)
+            if arrayMeta is not None:
+                arrayMeta['location'] = 'pointData'
+                arrayMeta['registration'] = 'addArray'
+                extractedFields.append(arrayMeta)
+
+        scaleOrientationArrayName = parent.GetInputArrayInformation(parent.ORIENTATION).Get(vtkDataObject.FIELD_NAME())
+        if scaleOrientationArrayName is not None and scaleOrientationArrayName not in [field['name'] for field in extractedFields]:
+            arrayMeta = getArrayDescription(dataset.GetPointData().GetAbstractArray(scaleOrientationArrayName), context)
+            if arrayMeta is not None:
+                arrayMeta['location'] = 'pointData'
+                arrayMeta['registration'] = 'addArray'
+                extractedFields.append(arrayMeta)
 
     # Normal handling
     if 'Normals' in requestedFields:
@@ -537,6 +581,14 @@ def extractRequiredFields(extractedFields, parent, dataset, context, requestedFi
 # Concrete instance serializers
 # -----------------------------------------------------------------------------
 
+def annotationSerializer(parent, prop, propId, context, depth):
+    if context.debugSerializers:
+        print('%s!!!Annotations are not handled directly by vtk.js but by bokeh model' % pad(depth))
+
+    context.addAnnotation(parent, prop, propId)
+
+    return None 
+
 def genericPropSerializer(parent, prop, popId, context, depth):
     # This kind of actor has two "children" of interest, a property and a
     # mapper (optionnaly a texture)
@@ -677,25 +729,25 @@ def genericMapperSerializer(parent, mapper, mapperId, context, depth):
     calls = []
     dependencies = []
 
-    if hasattr(mapper, 'GetInputDataObject'):
-        dataObject = mapper.GetInputDataObject(0, 0)
-    else:
+    if not hasattr(mapper, 'GetInputDataObject'):
         if context.debugAll:
-            print('This mapper does not have GetInputDataObject method')
-
-    if dataObject:
-        dataObjectId = '%s-dataset' % mapperId
-        if parent.IsA('vtkActor') and not mapper.IsA('vtkTexture'):
-            # vtk-js actors can render only surfacic datasets
-            # => we ensure to convert the dataset in polydata
-            dataObjectInstance = mergeToPolydataSerializer(
-                mapper, dataObject, dataObjectId, context, depth + 1)
-        else:
-            dataObjectInstance = serializeInstance(
-                mapper, dataObject, dataObjectId, context, depth + 1)
-        if dataObjectInstance:
-            dependencies.append(dataObjectInstance)
-            calls.append(['setInputData', [wrapId(dataObjectId)]])
+                print('This mapper does not have GetInputDataObject method')
+    else:
+        for port in range(mapper.GetNumberOfInputPorts()): # Glyph3DMapper can define input data objects on 2 ports (input, source)
+            dataObject = mapper.GetInputDataObject(port, 0)
+            if dataObject:
+                dataObjectId = '%s-dataset-%d' % (mapperId, port)
+                if parent.IsA('vtkActor') and not mapper.IsA('vtkTexture'):
+                    # vtk-js actors can render only surfacic datasets
+                    # => we ensure to convert the dataset in polydata
+                    dataObjectInstance = mergeToPolydataSerializer(
+                        mapper, dataObject, dataObjectId, context, depth + 1)
+                else:
+                    dataObjectInstance = serializeInstance(
+                        mapper, dataObject, dataObjectId, context, depth + 1)
+                if dataObjectInstance:
+                    dependencies.append(dataObjectInstance)
+                    calls.append(['setInputData', [wrapId(dataObjectId), port]])
 
     lookupTable = None
 
@@ -771,6 +823,25 @@ def genericVolumeMapperSerializer(parent, mapper, mapperId, context, depth):
 # -----------------------------------------------------------------------------
 
 
+def glyph3DMapperSerializer(parent, mapper, mapperId, context, depth):
+    instance = genericPolyDataMapperSerializer(parent, mapper, mapperId, context, depth)
+
+    if not instance: return
+    instance['type'] = mapper.GetClassName()
+    instance['properties'].update({
+        'orient': mapper.GetOrient(),
+        'orientationMode': mapper.GetOrientationMode(),
+        'scaling': mapper.GetScaling(),
+        'scaleFactor': mapper.GetScaleFactor(),
+        'scaleMode': mapper.GetScaleMode(),
+        'scaleArray': mapper.GetInputArrayInformation(mapper.SCALE).Get(vtkDataObject.FIELD_NAME()),
+        'orientationArray': mapper.GetInputArrayInformation(mapper.ORIENTATION).Get(vtkDataObject.FIELD_NAME()),
+    })
+    return instance
+
+# -----------------------------------------------------------------------------
+
+
 def textureSerializer(parent, texture, textureId, context, depth):
     instance = genericMapperSerializer(parent, texture, textureId, context, depth)
 
@@ -1092,7 +1163,7 @@ def mergeToPolydataSerializer(parent, dataObject, dataObjectId, context, depth):
         gf.Update()
         dataset = gf.GetOutput()
     else:
-        dataset = parent.GetInput()
+        dataset = dataObject
 
     return polydataSerializer(parent, dataset, dataObjectId, context, depth)
 
