@@ -16,5 +16,6 @@ def test_spacer(document, comm):
 
 def test_spacer_clone():
     spacer = Spacer(width=400, height=300)
-
-    assert spacer.param.get_param_values() == spacer.clone().param.get_param_values()
+    clone = spacer.clone()
+    assert ({k: v for k, v in spacer.param.get_param_values() if k != 'name'} ==
+            {k: v for k, v in clone.param.get_param_values() if k != 'name'})
