@@ -1,5 +1,3 @@
-from __future__ import absolute_import, division, unicode_literals
-
 import os
 import shutil
 
@@ -9,7 +7,7 @@ import pytest
 
 from panel.widgets import FileSelector
 
-@pytest.yield_fixture
+@pytest.fixture
 def test_dir():
     test_dir = os.path.expanduser(os.path.join('~','test_dir'))
     os.mkdir(test_dir)
