@@ -1,5 +1,3 @@
-from __future__ import absolute_import, division, unicode_literals
-
 from panel.models import Player as BkPlayer
 from panel.widgets import DiscretePlayer
 
@@ -17,7 +15,7 @@ def test_discrete_player(document, comm):
     assert widget.step == 1
 
     widget.value = 2
-    discrete_player._comm_change({'value': 2})
+    discrete_player._process_events({'value': 2})
     assert discrete_player.value == 10
 
     discrete_player.value = 100
