@@ -20,7 +20,7 @@ from bokeh.models.widgets.tables import (
 )
 
 from panel.depends import bind
-from panel.widgets import DataFrame, Tabulator, TextInput
+from panel.widgets import Button, DataFrame, Tabulator, TextInput
 
 pd_old = pytest.mark.skipif(LooseVersion(pd.__version__) < '1.3',
                           reason="Requires latest pandas")
@@ -205,6 +205,19 @@ def test_hierarchical_index(document, comm):
     assert isinstance(agg2, MinAggregator)
 
 
+def test_table_index_column(document, comm):
+    df = pd.DataFrame({
+        'int': [1, 2, 3],
+        'float': [3.14, 6.28, 9.42],
+        'index': ['A', 'B', 'C'],
+    }, index=[1, 2, 3])
+    table = DataFrame(value=df)
+
+    model = table.get_root(document, comm=comm)
+
+    assert np.array_equal(model.source.data['level_0'], np.array([1, 2, 3]))
+    assert model.columns[0].field == 'level_0'
+    assert model.columns[0].title == ''
 
 
 def test_none_table(document, comm):
@@ -223,6 +236,40 @@ def test_tabulator_selected_dataframe():
     pd.testing.assert_frame_equal(table.selected_dataframe, df.iloc[[0, 2]])
 
 
+def test_tabulator_multi_index(document, comm):
+    df = makeMixedDataFrame()
+    table = Tabulator(df.set_index(['A', 'C']))
+
+    model = table.get_root()
+
+    assert model.configuration['columns'] == [
+        {'field': 'A'},
+        {'field': 'C'},
+        {'field': 'B'},
+        {'field': 'D'}
+    ]
+
+    assert np.array_equal(model.source.data['A'], np.array([0., 1., 2., 3., 4.]))
+    assert np.array_equal(model.source.data['C'], np.array(['foo1', 'foo2', 'foo3', 'foo4', 'foo5']))
+
+
+def test_tabulator_multi_index_remote_pagination(document, comm):
+    df = makeMixedDataFrame()
+    table = Tabulator(df.set_index(['A', 'C']), pagination='remote', page_size=3)
+
+    model = table.get_root()
+
+    assert model.configuration['columns'] == [
+        {'field': 'A'},
+        {'field': 'C'},
+        {'field': 'B'},
+        {'field': 'D'}
+    ]
+
+    assert np.array_equal(model.source.data['A'], np.array([0., 1., 2.]))
+    assert np.array_equal(model.source.data['C'], np.array(['foo1', 'foo2', 'foo3']))
+
+
 def test_tabulator_selected_and_filtered_dataframe(document, comm):
     df = makeMixedDataFrame()
     table = Tabulator(df)
@@ -307,6 +354,23 @@ def test_tabulator_groups(document, comm):
     ]
 
 
+def test_tabulator_numeric_groups(document, comm):
+    df = pd.DataFrame(np.random.rand(10, 3))
+    table = Tabulator(df, groups={'Number': [0, 1]})
+
+    model = table.get_root(document, comm)
+
+    assert model.configuration['columns'] == [
+        {'field': 'index'},
+        {'title': 'Number',
+         'columns': [
+            {'field': '0'},
+            {'field': '1'}
+        ]},
+        {'field': '2'}
+    ]
+
+
 def test_tabulator_frozen_cols(document, comm):
     df = makeMixedDataFrame()
     table = Tabulator(df, frozen_columns=['index'])
@@ -957,3 +1021,34 @@ def test_tabulator_dataframe_replace_data(document, comm):
     }
     for col, values in model.source.data.items():
         np.testing.assert_array_equal(values, expected[col])
+
+
+def test_tabulator_download_menu_default():
+    df = makeMixedDataFrame()
+    table = Tabulator(df)
+
+    filename, button = table.download_menu()
+
+    assert isinstance(filename, TextInput)
+    assert isinstance(button, Button)
+
+    assert filename.value == 'table.csv'
+    assert filename.name == 'Filename'
+    assert button.name == 'Download'
+
+
+def test_tabulator_download_menu_custom_kwargs():
+    df = makeMixedDataFrame()
+    table = Tabulator(df)
+
+    filename, button = table.download_menu(
+        text_kwargs={'name': 'Enter filename', 'value': 'file.csv'},
+        button_kwargs={'name': 'Download table'},
+    )
+
+    assert isinstance(filename, TextInput)
+    assert isinstance(button, Button)
+
+    assert filename.value == 'file.csv'
+    assert filename.name == 'Enter filename'
+    assert button.name == 'Download table'
