@@ -1,6 +1,7 @@
 """
 Various general utilities used in the panel codebase.
 """
+import ast
 import base64
 import datetime as dt
 import inspect
@@ -293,7 +294,12 @@ def parse_query(query):
         elif is_number(v):
             query[k] = float(v)
         elif v.startswith('[') or v.startswith('{'):
-            query[k] = json.loads(v)
+            try:
+                query[k] = json.loads(v)
+            except Exception:
+                query[k] = ast.literal_eval(v)
+        elif v.lower() in ("true", "false"):
+            query[k] = v.lower() == "true"
     return query
 
 
@@ -355,16 +361,27 @@ def edit_readonly(parameterized):
             p.constant = constant
 
 
-def lazy_load(module, model, notebook=False):
+def lazy_load(module, model, notebook=False, root=None):
     if module in sys.modules:
         return getattr(sys.modules[module], model)
     if notebook:
         ext = module.split('.')[-1]
-        param.main.param.warning(f'{model} was not imported on instantiation '
-                                 'and may not render in a notebook. Restart '
-                                 'the notebook kernel and ensure you load '
-                                 'it as part of the extension using:'
-                                 f'\n\npn.extension(\'{ext}\')\n')
+        param.main.param.warning(
+            f'{model} was not imported on instantiation and may not '
+            'render in a notebook. Restart the notebook kernel and '
+            'ensure you load it as part of the extension using:'
+            f'\n\npn.extension(\'{ext}\')\n'
+        )
+    elif root is not None:
+        from .io.state import state
+        ext = module.split('.')[-1]
+        if root.ref['id'] in state._views:
+            param.main.param.warning(
+                f'{model} was not imported on instantiation may not '
+                'render in the served application. Ensure you add the '
+                'following to the top of your application:'
+                f'\n\npn.extension(\'{ext}\')\n'
+            )
     return getattr(import_module(module), model)
 
 
