@@ -19,9 +19,6 @@ class Widget(Reactive):
     disabled = param.Boolean(default=False, doc="""
        Whether the widget is disabled.""")
 
-    visible = param.Boolean(default=True, doc="""
-       Whether the widget is visible or not.""")
-
     name = param.String(default='')
 
     height = param.Integer(default=None, bounds=(0, None))
@@ -133,13 +130,14 @@ class CompositeWidget(Widget):
 
     def __init__(self, **params):
         super().__init__(**params)
-        layout = {p: getattr(self, p) for p in Layoutable.param
+        layout_params = [p for p in Layoutable.param if p != 'name']
+        layout = {p: getattr(self, p) for p in layout_params
                   if getattr(self, p) is not None}
         if layout.get('width', self.width) is None and not 'sizing_mode' in layout:
             layout['sizing_mode'] = 'stretch_width'
         self._composite = self._composite_type(**layout)
         self._models = self._composite._models
-        self.param.watch(self._update_layout_params, list(Layoutable.param))
+        self.param.watch(self._update_layout_params, layout_params)
 
     def _update_layout_params(self, *events):
         updates = {event.name: event.new for event in events}
